<?php

use CRM_Phoneinputmask_ExtensionUtil as E;

/**
 * Form controller class
 *
 * @see https://docs.civicrm.org/dev/en/latest/framework/quickform/
 */
class CRM_Phoneinputmask_Form_PhoneInputMask extends CRM_Core_Form {

  protected $_id;

  protected $_phoneinputmask;

  private $snippet;

  /**
   * @var \Civi\PhoneInputMask\TypeInterface
   */
  private $typeClass;

  /**
   * @var \Civi\PhoneInputMask\TypeInterface[]
   */
  private $types;

  public function getDefaultEntity() {
    return 'PhoneInputMask';
  }

  public function getDefaultEntityTable() {
    return 'civicrm_phone_input_mask';
  }

  public function getEntityId() {
    return $this->_id;
  }

  /**
   * Preprocess form.
   *
   * This is called before buildForm. Any pre-processing that
   * needs to be done for buildForm should be done here.
   *
   * This is a virtual function and should be redefined if needed.
   */
  public function preProcess() {
    $factory = phoneinputmask_get_factory();
    $this->types = $factory->getTypes();
    $this->snippet = CRM_Utils_Request::retrieve('snippet', 'String');
    if ($this->snippet) {
      $this->assign('suppressForm', TRUE);
      $this->controller->_generateQFKey = FALSE;
    }

    parent::preProcess();

    $this->_action = CRM_Utils_Request::retrieve('action', 'String', $this);
    $this->assign('action', $this->_action);

    $this->_id = CRM_Utils_Request::retrieve('id', 'Positive', $this, FALSE);

    CRM_Utils_System::setTitle('Add Phone Input Mask');
    if ($this->_id) {
      CRM_Utils_System::setTitle('Edit Phone Input Mask');
      $this->_phoneinputmask = \Civi\Api4\PhoneInputMask::get()->addWhere('id', '=', $this->_id)->setLimit(1)->execute()->first();
      $this->assign('phoneinputmask', $this->_phoneinputmask);
    } else {
      $this->_phoneinputmask['is_active'] = '1';
    }

    $type = CRM_Utils_Request::retrieve('type', 'String');
    if (!$type && $this->_phoneinputmask && isset($this->_phoneinputmask['type'])) {
      $type = $this->_phoneinputmask['type'];
    }
    if ($type) {
      $this->typeClass = $this->types[$type];
      $this->assign('has_configuration', $this->typeClass->hasConfiguration());
    }
  }

  protected function getTypeList() {
    $return = [];
    foreach($this->types as $type => $typeClass) {
      $return[$type] = $typeClass->getTypeTitle();
    }
    return $return;
  }

  public function buildQuickForm() {
    $this->assign('id', $this->getEntityId());
    $this->add('hidden', 'id');
    if ($this->_action != CRM_Core_Action::DELETE) {
      $this->add('text', 'title', E::ts('Title'), ['class' => 'huge'], true);
      $this->add('text', 'name', E::ts('Name'), ['class' => 'huge'], false);
      $this->add('checkbox', 'is_active', E::ts('Enabled'));

      $this->add('select', 'type', E::ts('Validation Type'), $this->getTypeList(), true, array(
        'style' => 'min-width:250px',
        'class' => 'crm-select2 huge',
        'placeholder' => E::ts('- select -'),
      ));

      if ($this->typeClass && $this->typeClass->hasConfiguration()) {
        $this->typeClass->buildConfigurationForm($this, $this->_phoneinputmask);
        $this->assign('configuration_template', $this->typeClass->getConfigurationTemplateFileName());
      }

      $this->addButtons([
        [
          'type' => 'upload',
          'name' => E::ts('Save'),
          'isDefault' => TRUE,
        ],
      ]);
    } else {
      $this->addButtons([
        ['type' => 'submit', 'name' => E::ts('Delete'), 'isDefault' => TRUE],
        ['type' => 'cancel', 'name' => E::ts('Cancel')]
      ]);
    }
    parent::buildQuickForm();
  }

  /**
   * Function to add validation rules (overrides parent function)
   *
   * @access public
   */
  function addRules() {
    if ($this->_action != CRM_Core_Action::DELETE) {
      $this->addFormRule(array(
        'CRM_Phoneinputmask_Form_PhoneInputMask',
        'validateName'
      ));
    }
  }


  /**
   * This virtual function is used to set the default values of various form
   * elements.
   *
   * @return array|NULL
   *   reference to the array of default values
   */
  public function setDefaultValues() {
    $defaults = array();
    if ($this->_phoneinputmask) {
      $defaults = $this->_phoneinputmask;
    }
    return $defaults;
  }

  public function postProcess() {
    if ($this->_action == CRM_Core_Action::DELETE) {
      civicrm_api4('PhoneInputMask', 'delete', ['where' => [['id', '=', $this->_id]]]);
      CRM_Core_Session::setStatus(E::ts('Removed Phone Input Mask'), E::ts('Phone Input Mask'), 'success');
    } else {
      $values = $this->controller->exportValues();
      $action = 'create';
      if ($this->getEntityId()) {
        $params['id'] = $this->getEntityId();
        $action = 'update';
      }
      $params['title'] = $values['title'];
      $params['name'] = $values['name'];
      $params['type'] = $values['type'];
      $params['is_active'] = $values['is_active'] ? '1' : '0';
      if ($this->typeClass && $this->typeClass->hasConfiguration()) {
        $params['configuration'] = $this->typeClass->processConfiguration($values);
      }
      civicrm_api4('PhoneInputMask', $action, ['values' => $params]);
    }

    $session = CRM_Core_Session::singleton();
    $url = $session->popUserContext();
    CRM_Utils_System::redirect($url);

    parent::postProcess();
  }

  /**
   * Function to validate if a certain name already exists
   *
   * @param array $fields
   * @return array|bool
   * @access static
   */
  static function validateName($fields) {
    /*
     * if id not empty, edit mode. Check if changed before check if exists
     */
    $id = false;
    if (!empty($fields['id'])) {
      $id = $fields['id'];
    }
    if (empty($fields['name'])) {
      $fields['name'] = CRM_Phoneinputmask_BAO_PhoneInputMask::checkName($fields['title'], $id);
    }
    if (!CRM_Phoneinputmask_BAO_PhoneInputMask::isNameValid($fields['name'], $id)) {
      $errors['name'] = E::ts('There is already a phone input mask with this name');
      return $errors;
    }
    return TRUE;
  }


}
