<?php
/**
 * Copyright (C) 2021  Jaap Jansma (jaap.jansma@civicoop.org)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

use Civi\Test\HeadlessInterface;
use Civi\Test\HookInterface;
use Civi\Test\TransactionalInterface;

/**
 * PhoneInputMask API Test Case
 * @group headless
 */
class CRM_Phoneinputmask_Utils_ValidateMaskTest extends \PHPUnit\Framework\TestCase implements HeadlessInterface, HookInterface, TransactionalInterface {

  use \Civi\Test\Api3TestTrait;

  /**
   * Set up for headless tests.
   *
   * Civi\Test has many helpers, like install(), uninstall(), sql(), and sqlFile().
   *
   * See: https://docs.civicrm.org/dev/en/latest/testing/phpunit/#civitest
   */
  public function setUpHeadless() {
    return \Civi\Test::headless()
      ->installMe(__DIR__)
      ->apply();
  }

  /**
   * @dataProvider provideTestData
   */
  public function testValidation($masks, $validPhoneNumbers, $invalidPhoneNumbers) {
    foreach($masks as $mask) {
      $this->callAPISuccess('PhoneInputMask', 'create', $mask);
    }
    foreach($validPhoneNumbers as $validPhoneNumber) {
      $result = CRM_Phoneinputmask_Utils_CommonForm::validatePhone($validPhoneNumber);
      $this->assertTrue($result, 'Phone number: "'  . $validPhoneNumber. '" should be valid');
    }

    foreach($invalidPhoneNumbers as $invalidPhoneNumber) {
      $result = CRM_Phoneinputmask_Utils_CommonForm::validatePhone($invalidPhoneNumber);
      $this->assertFalse($result, 'Phone number: "'  . $invalidPhoneNumber. '" should be invalid');
    }
  }

  public function provideTestData() {
    return [
      'Active and Inactive' => [
        [
          [
            'title' => 'Test Phone Mask',
            'type' => 'mask',
            'configuration' => [
              'mask' => '(9900)-00000099',
              'example' => '(0012)-12345678'
            ],
            'is_active' => 1,
          ],
          [
            'title' => 'Inactive Phone Mask',
            'type' => 'mask',
            'configuration' => [
              'mask' => '00-00000000',
              'example' => '12-34567890'
            ],
            'is_active' => 0,
          ]
        ],
        [
          '(0123)-123456',
        ],
        [
          '01-231234567',
        ]
      ],
      'Test 9999999999' => [
        [
          [
            'title' => 'Test Phone Mask',
            'type' => 'mask',
            'configuration' => [
              'mask' => '9999999999',
              'example' => '0112345678'
            ],
            'is_active' => 1,
          ],
        ],
        [
          '0112345678',
          '012345678',
          '1'
        ],
        [
          '0123 45678',
          '0123-45678',
          '0123 - 45678',
          '(0123) 45678',
          'hello world',
          'helloworld',
          '0123hello',
          'hello0123',
          '012abc456'
        ]
      ],
      'Test NL numbers (0099)###0000099' => [
        [
          [
            'title' => 'Test Phone Mask',
            'type' => 'mask',
            'configuration' => [
              'mask' => '(0099)###00000099',
              'example' => '(0012) - 1234567'
            ],
            'is_active' => 1,
          ],
        ],
        [
          '(0123) - 123456',
          '(0123) - 1234567',
          '(012) - 123456',
          '(06) - 12345678',
        ],
        [
          '1',
          '(0123) 45678',
          '(0123)-45678',
          '0112345678',
          '012345678',
          '0123 45678',
          '0123-45678',
          '0123 - 45678',
          'hello world',
          'helloworld',
          '0123hello',
          'hello0123',
          '012abc456'
        ]
      ],
      'Test DUDEN' => [
        [
          [
            'title' => 'Test Phone Mask',
            'type' => 'duden',
            'configuration' => [],
            'is_active' => 1,
          ],
        ],
        [
          '0 30 / 1 23 45 67',
          '0 30 / 12 34 56',
          '0 3045 / 12 34 56',
          '0 12 34 / 1 23 45 67',
          '01 23 / 12 34',
          '0 12 / 1 23 45'
        ],
        [
          '0 30 / 1234 56',
          '0 30 / 12 34 56 7',
          '0 30 / 12 3 45 67',
          '0 30 / 123 456',
          '0 30 / 123456',
          '1',
          '(0123) 45678',
          '(0123)-45678',
          '0112345678',
          '+49 3444 1234 567',
          '012345678',
          '0123 45678',
          '0123-45678',
          '0123 - 45678',
          'hello world',
          'helloworld',
          '0123hello',
          'hello0123',
          '012abc456'
        ]
      ],
      'Test DIN 5008' => [
        [
          [
            'title' => 'DIN5008',
            'type' => 'din5008',
            'configuration' => [],
            'is_active' => 1,
          ]
        ],
        [
          '+49 1299 12349999',
          '+49 1299 1234',
          '+49 1299 12345',
          '+49 12 12349999',
          '+49 123 12349999',
          '+49 1299 12349999-12',
          '+49 1299 1234-12',
          '+49 1299 12345-12',
          '+49 12 12349999-12',
          '+49 123 12349999-12',
          '01299 12349999',
          '01299 12349',
          '012 12349999',
          '012 1234',
          '01299 12345',
          '0123 1234',
          '0123 12345',
          '01299 12349999-99',
          '01299 12349-12',
          '012 12349999-12',
          '012 1234-12',
          '01299 12345-12',
          '0123 1234-12',
          '0123 12345-12',
        ],
        [
          '+49 1299 123499998',
          '+49 1299 12349999-123',
          '+49 1299 123',
          '+49 1 12349999',
          '11299 12349999',
          '01299 12349999-123',
          '1299 12349999',
        ]
      ],
      'Test E.123' => [
        [
          [
            'title' => 'E123',
            'type' => 'e123',
            'configuration' => [],
            'is_active' => 1,
          ]
        ],
        [
          '+31 42 123 4567',
          '(112) 123 4567',
          '(030) 222222',
          '+44 1234 567890',
          '+44 1234 5678908',
          '(089) 123 4567',
          '(089) 1234567',
          '(0852) 1234567',
          '(0852) 123 4567',
          '(06104) 1234567',
          '(06104) 123 4567',
          '+49 5822 123 4567',
          '+49 5822 1234567',
        ],
        [
          '+31 42 123 45678',
          '(112) 123 45678',
        ]
      ]
    ];
  }

}
